@@ -156,13 +156,22 @@ static void pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
 
 	seat_set_outputs_dirty(seat);
 
-	wl_surface_set_buffer_scale(seat->cursor_surface, output->scale);
-	wl_surface_attach(seat->cursor_surface,
+	if (output->state->cursor_shape_manager) {
+		struct wp_cursor_shape_device_v1 *device =
+			wp_cursor_shape_manager_v1_get_pointer(
+				output->state->cursor_shape_manager, wl_pointer);
+		wp_cursor_shape_device_v1_set_shape(device, serial,
+			WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CROSSHAIR);
+		wp_cursor_shape_device_v1_destroy(device);
+	} else {
+		wl_surface_set_buffer_scale(seat->cursor_surface, output->scale);
+		wl_surface_attach(seat->cursor_surface,
 			wl_cursor_image_get_buffer(output->cursor_image), 0, 0);
-	wl_pointer_set_cursor(wl_pointer, serial, seat->cursor_surface,
+		wl_pointer_set_cursor(wl_pointer, serial, seat->cursor_surface,
 			output->cursor_image->hotspot_x / output->scale,
 			output->cursor_image->hotspot_y / output->scale);
-	wl_surface_commit(seat->cursor_surface);
+		wl_surface_commit(seat->cursor_surface);
+	}
 }
 
 static void pointer_handle_leave(void *data, struct wl_pointer *wl_pointer,
@@ -818,6 +827,45 @@ static void add_choice_box(struct slurp_state *state,
 	wl_list_insert(state->boxes.prev, &b->link);
 }
 
+static bool create_cursors(struct slurp_state *state) {
+	const char *cursor_theme = getenv("XCURSOR_THEME");
+	const char *cursor_size_str = getenv("XCURSOR_SIZE");
+	int cursor_size = 24;
+	if (cursor_size_str != NULL) {
+		char *end;
+		errno = 0;
+		cursor_size = strtol(cursor_size_str, &end, 10);
+		if (errno != 0 || cursor_size_str[0] == '\0' || end[0] != '\0') {
+			fprintf(stderr, "invalid XCURSOR_SIZE value\n");
+			return false;
+		}
+	}
+
+	struct slurp_output *output;
+	wl_list_for_each(output, &state->outputs, link) {
+		output->cursor_theme = wl_cursor_theme_load(cursor_theme,
+			cursor_size * output->scale, state->shm);
+		if (output->cursor_theme == NULL) {
+			fprintf(stderr, "failed to load cursor theme\n");
+			return false;
+		}
+		struct wl_cursor *cursor =
+			wl_cursor_theme_get_cursor(output->cursor_theme, "crosshair");
+		if (cursor == NULL) {
+			// Fallback
+			cursor =
+				wl_cursor_theme_get_cursor(output->cursor_theme, "left_ptr");
+		}
+		if (cursor == NULL) {
+			fprintf(stderr, "failed to load cursor\n");
+			return false;
+		}
+		output->cursor_image = cursor->images[0];
+	}
+
+	return true;
+}
+
 int main(int argc, char *argv[]) {
 	int status = EXIT_SUCCESS;
 
@@ -967,19 +1015,6 @@ int main(int argc, char *argv[]) {
 		return EXIT_FAILURE;
 	}
 
-	const char *cursor_theme = getenv("XCURSOR_THEME");
-	const char *cursor_size_str = getenv("XCURSOR_SIZE");
-	int cursor_size = 24;
-	if (cursor_size_str != NULL) {
-		char *end;
-		errno = 0;
-		cursor_size = strtol(cursor_size_str, &end, 10);
-		if (errno != 0 || cursor_size_str[0] == '\0' || end[0] != '\0') {
-			fprintf(stderr, "invalid XCURSOR_SIZE value\n");
-			return EXIT_FAILURE;
-		}
-	}
-
 	struct slurp_output *output;
 	wl_list_for_each(output, &state.outputs, link) {
 		output->surface = wl_compositor_create_surface(state.compositor);
@@ -1011,29 +1046,14 @@ int main(int argc, char *argv[]) {
 		zwlr_layer_surface_v1_set_keyboard_interactivity(output->layer_surface, true);
 		zwlr_layer_surface_v1_set_exclusive_zone(output->layer_surface, -1);
 		wl_surface_commit(output->surface);
-
-		output->cursor_theme = wl_cursor_theme_load(cursor_theme,
-			cursor_size * output->scale, state.shm);
-		if (output->cursor_theme == NULL) {
-			fprintf(stderr, "failed to load cursor theme\n");
-			return EXIT_FAILURE;
-		}
-		struct wl_cursor *cursor =
-			wl_cursor_theme_get_cursor(output->cursor_theme, "crosshair");
-		if (cursor == NULL) {
-			// Fallback
-			cursor =
-				wl_cursor_theme_get_cursor(output->cursor_theme, "left_ptr");
-		}
-		if (cursor == NULL) {
-			fprintf(stderr, "failed to load cursor\n");
-			return EXIT_FAILURE;
-		}
-		output->cursor_image = cursor->images[0];
 	}
 	// second roundtrip for xdg-output
 	wl_display_roundtrip(state.display);
 
+	if (!state.cursor_shape_manager && !create_cursors(&state)) {
+		return EXIT_FAILURE;
+	}
+
 	if (output_boxes) {
 		struct slurp_output *box_output;
 		wl_list_for_each(box_output, &state.outputs, link) {
